@@ -1,6 +1,6 @@
 ;; Calculator for GNU Emacs, part I [calc-aent.el]
-;; Copyright (C) 1990, 1991 Free Software Foundation, Inc.
-;; Written by Dave Gillespie, daveg@csvax.cs.caltech.edu.
+;; Copyright (C) 1990, 1991, 1992, 1993 Free Software Foundation, Inc.
+;; Written by Dave Gillespie, daveg@synaptics.com.
 
 ;; This file is part of GNU Emacs.
 
@@ -207,6 +207,7 @@
 	    (and (memq 'clear-message calc-command-flags)
 		 (message ""))
 	    (cond ((eq separator 'pred)
+		   (calc-extensions)
 		   (if (= (length res) 1)
 		       (math-is-true (car res))
 		     (calc-eval-error '(0 "Single value expected"))))
@@ -306,15 +307,18 @@
       (define-key calc-alg-ent-map "\C-m" 'calcAlg-enter)
       (define-key calc-alg-ent-map "\C-j" 'calcAlg-enter)
       (setq calc-alg-ent-esc-map (copy-sequence esc-map))
-      (let ((i 33))
-	(while (< i 127)
-	  (aset calc-alg-ent-esc-map i 'calcAlg-escape)
-	  (setq i (1+ i)))))
+      (or (string-match "^19" emacs-version)
+	  (let ((i 33))
+	    (while (< i 127)
+	      (aset calc-alg-ent-esc-map i 'calcAlg-escape)
+	      (setq i (1+ i))))))
     (define-key calc-alg-ent-map "\e" nil)
     (if (eq calc-algebraic-mode 'total)
 	(define-key calc-alg-ent-map "\e" calc-alg-ent-esc-map)
       (define-key calc-alg-ent-map "\ep" 'calcAlg-plus-minus)
       (define-key calc-alg-ent-map "\em" 'calcAlg-mod)
+      (define-key calc-alg-ent-map "\e=" 'calcAlg-equals)
+      (define-key calc-alg-ent-map "\e\r" 'calcAlg-equals)
       (define-key calc-alg-ent-map "\e%" 'self-insert-command))
     (setq calc-aborted-prefix nil)
     (let ((buf (read-from-minibuffer (or prompt "Algebraic: ")
@@ -356,6 +360,15 @@
     (insert "'"))
 )
 
+(defun calcAlg-equals ()
+  (interactive)
+  (unwind-protect
+      (calcAlg-enter)
+    (if (consp alg-exp)
+	(setq prefix-arg (length alg-exp)
+	      unread-command-char ?=)))
+)
+
 (defun calcAlg-escape ()
   (interactive)
   (setq unread-command-char last-command-char)
@@ -462,9 +475,10 @@
 	exp-token exp-data)
     (if calc-language-input-filter
 	(setq exp-str (funcall calc-language-input-filter exp-str)))
-    (while (setq exp-token (string-match "\\.\\." exp-str))
+    (while (setq exp-token (string-match "\\.\\.\\([^.]\\|.[^.]\\)" exp-str))
       (setq exp-str (concat (substring exp-str 0 exp-token) "\\dots"
 			    (substring exp-str (+ exp-token 2)))))
+    (math-build-parse-table)
     (math-read-token)
     (let ((val (catch 'syntax (math-read-expr-list))))
       (if (stringp val)
@@ -486,6 +500,59 @@
     val)
 )
 
+(setq calc-user-parse-table nil)
+(setq calc-last-main-parse-table nil)
+(setq calc-last-lang-parse-table nil)
+(setq calc-user-tokens nil)
+(setq calc-user-token-chars nil)
+
+(defun math-build-parse-table ()
+  (let ((mtab (cdr (assq nil calc-user-parse-tables)))
+	(ltab (cdr (assq calc-language calc-user-parse-tables))))
+    (or (and (eq mtab calc-last-main-parse-table)
+	     (eq ltab calc-last-lang-parse-table))
+	(let ((p (append mtab ltab))
+	      (toks nil))
+	  (setq calc-user-parse-table p)
+	  (setq calc-user-token-chars nil)
+	  (while p
+	    (math-find-user-tokens (car (car p)))
+	    (setq p (cdr p)))
+	  (setq calc-user-tokens (mapconcat 'identity
+					    (sort (mapcar 'car toks)
+						  (function (lambda (x y)
+							      (> (length x)
+								 (length y)))))
+					    "\\|")
+		calc-last-main-parse-table mtab
+		calc-last-lang-parse-table ltab))))
+)
+
+(defun math-find-user-tokens (p)   ; uses "toks"
+  (while p
+    (cond ((and (stringp (car p))
+		(or (> (length (car p)) 1) (equal (car p) "$")
+		    (equal (car p) "\""))
+		(string-match "[^a-zA-Z0-9]" (car p)))
+	   (let ((s (regexp-quote (car p))))
+	     (if (string-match "\\`[a-zA-Z0-9]" s)
+		 (setq s (concat "\\<" s)))
+	     (if (string-match "[a-zA-Z0-9]\\'" s)
+		 (setq s (concat s "\\>")))
+	     (or (assoc s toks)
+		 (progn
+		   (setq toks (cons (list s) toks))
+		   (or (memq (aref (car p) 0) calc-user-token-chars)
+		       (setq calc-user-token-chars
+			     (cons (aref (car p) 0)
+				   calc-user-token-chars)))))))
+	  ((consp (car p))
+	   (math-find-user-tokens (nth 1 (car p)))
+	   (or (eq (car (car p)) '\?)
+	       (math-find-user-tokens (nth 2 (car p))))))
+    (setq p (cdr p)))
+)
+
 (defun math-read-token ()
   (if (>= exp-pos (length exp-str))
       (setq exp-old-pos exp-pos
@@ -499,6 +566,13 @@
 		 (setq exp-token 'space
 		       exp-data " ")
 	       (math-read-token)))
+	    ((and (memq ch calc-user-token-chars)
+		  (let ((case-fold-search nil))
+		    (eq (string-match calc-user-tokens exp-str exp-pos)
+			exp-pos)))
+	     (setq exp-token 'punc
+		   exp-data (math-match-substring exp-str 0)
+		   exp-pos (match-end 0)))
 	    ((or (and (>= ch ?a) (<= ch ?z))
 		 (and (>= ch ?A) (<= ch ?Z)))
 	     (string-match (if (memq calc-language '(c fortran pascal maple))
@@ -586,7 +660,8 @@
 	       (setq exp-token 'string
 		     exp-data (math-match-substring exp-str 1)
 		     exp-pos (match-end 0))))
-	    ((and (= ch ?\\) (eq calc-language 'tex))
+	    ((and (= ch ?\\) (eq calc-language 'tex)
+		  (< exp-pos (1- (length exp-str))))
 	     (or (string-match "\\\\hbox *{\\([a-zA-Z0-9]+\\)}" exp-str exp-pos)
 		 (string-match "\\(\\\\\\([a-zA-Z]+\\|[^a-zA-Z]\\)\\)" exp-str exp-pos))
 	     (setq exp-token 'symbol
@@ -648,9 +723,13 @@
 )
 
 
-(defun math-read-expr-level (exp-prec)
+(defun math-read-expr-level (exp-prec &optional exp-term)
   (let* ((x (math-read-factor)) (first t) op op2)
-    (while (and (or (and (setq op (assoc exp-data math-expr-opers))
+    (while (and (or (and calc-user-parse-table
+			 (setq op (calc-check-user-syntax x exp-prec))
+			 (setq x op
+			       op '("2x" ident 999999 -1)))
+		    (and (setq op (assoc exp-data math-expr-opers))
 			 (/= (nth 2 op) -1)
 			 (or (and (setq op2 (assoc
 					     exp-data
@@ -668,7 +747,21 @@
 				  (not (eq calc-language 'math))
 				  (not (and exp-keep-spaces
 					    (eq (car-safe x) 'vec)))))
+			 (or (not (setq op (assoc exp-data math-expr-opers)))
+			     (/= (nth 2 op) -1))
+			 (or (not calc-user-parse-table)
+			     (not (eq exp-token 'symbol))
+			     (let ((p calc-user-parse-table))
+			       (while (and p
+					   (or (not (integerp
+						     (car (car (car p)))))
+					       (not (equal
+						     (nth 1 (car (car p)))
+						     exp-data))))
+				 (setq p (cdr p)))
+			       (not p)))
 			 (setq op (assoc "2x" math-expr-opers))))
+		(not (and exp-term (equal exp-data exp-term)))
 		(>= (nth 2 op) exp-prec))
       (if (not (equal (car op) "2x"))
 	  (math-read-token))
@@ -693,11 +786,158 @@
 		     (math-composite-inequalities x op))
 		    (t (list (nth 1 op)
 			     x
-			     (math-read-expr-level (nth 3 op)))))
+			     (math-read-expr-level (nth 3 op) exp-term))))
 	    first nil))
     x)
 )
 
+(defun calc-check-user-syntax (&optional x prec)
+  (let ((p calc-user-parse-table)
+	(matches nil)
+	match rule)
+    (while (and p
+		(or (not (progn
+			   (setq rule (car (car p)))
+			   (if x
+			       (and (integerp (car rule))
+				    (>= (car rule) prec)
+				    (equal exp-data
+					   (car (setq rule (cdr rule)))))
+			     (equal exp-data (car rule)))))
+		    (let ((save-exp-pos exp-pos)
+			  (save-exp-old-pos exp-old-pos)
+			  (save-exp-token exp-token)
+			  (save-exp-data exp-data))
+		      (or (not (listp
+				(setq matches (calc-match-user-syntax rule))))
+			  (let ((args (progn
+					(calc-extensions)
+					calc-arg-values))
+				(conds nil)
+				temp)
+			    (if x
+				(setq matches (cons x matches)))
+			    (setq match (cdr (car p)))
+			    (while (and (eq (car-safe match)
+					    'calcFunc-condition)
+					(= (length match) 3))
+			      (setq conds (append (math-flatten-lands
+						   (nth 2 match))
+						  conds)
+				    match (nth 1 match)))
+			    (while (and conds match)
+			      (calc-extensions)
+			      (cond ((eq (car-safe (car conds))
+					 'calcFunc-let)
+				     (setq temp (car conds))
+				     (or (= (length temp) 3)
+					 (and (= (length temp) 2)
+					      (eq (car-safe (nth 1 temp))
+						  'calcFunc-assign)
+					      (= (length (nth 1 temp)) 3)
+					      (setq temp (nth 1 temp)))
+					 (setq match nil))
+				     (setq matches (cons
+						    (math-normalize
+						     (math-multi-subst
+						      (nth 2 temp)
+						      args matches))
+						    matches)
+					   args (cons (nth 1 temp)
+						      args)))
+				    ((and (eq (car-safe (car conds))
+					      'calcFunc-matches)
+					  (= (length (car conds)) 3))
+				     (setq temp (calcFunc-vmatches
+						 (math-multi-subst
+						  (nth 1 (car conds))
+						  args matches)
+						 (nth 2 (car conds))))
+				     (if (eq temp 0)
+					 (setq match nil)
+				       (while (setq temp (cdr temp))
+					 (setq matches (cons (nth 2 (car temp))
+							     matches)
+					       args (cons (nth 1 (car temp))
+							  args)))))
+				    (t
+				     (or (math-is-true (math-simplify
+							(math-multi-subst
+							 (car conds)
+							 args matches)))
+					 (setq match nil))))
+			      (setq conds (cdr conds)))
+			    (if match
+				(not (setq match (math-multi-subst
+						  match args matches)))
+			      (setq exp-old-pos save-exp-old-pos
+				    exp-token save-exp-token
+				    exp-data save-exp-data
+				    exp-pos save-exp-pos)))))))
+      (setq p (cdr p)))
+    (and p match))
+)
+
+(defun calc-match-user-syntax (p &optional term)
+  (let ((matches nil)
+	(save-exp-pos exp-pos)
+	(save-exp-old-pos exp-old-pos)
+	(save-exp-token exp-token)
+	(save-exp-data exp-data))
+    (while (and p
+		(cond ((stringp (car p))
+		       (and (equal exp-data (car p))
+			    (progn
+			      (math-read-token)
+			      t)))
+		      ((integerp (car p))
+		       (and (setq m (catch 'syntax
+				      (math-read-expr-level
+				       (car p)
+				       (if (cdr p)
+					   (if (consp (nth 1 p))
+					       (car (nth 1 (nth 1 p)))
+					     (nth 1 p))
+					 term))))
+			    (not (stringp m))
+			    (setq matches (nconc matches (list m)))))
+		      ((eq (car (car p)) '\?)
+		       (setq m (calc-match-user-syntax (nth 1 (car p))))
+		       (or (nth 2 (car p))
+			   (setq matches
+				 (nconc matches
+					(list
+					 (cons 'vec (and (listp m) m))))))
+		       (or (listp m) (not (nth 2 (car p)))
+			   (not (eq (aref (car (nth 2 (car p))) 0) ?\$))
+			   (eq exp-token 'end)))
+		      (t
+		       (setq m (calc-match-user-syntax (nth 1 (car p))
+						       (car (nth 2 (car p)))))
+		       (if (listp m)
+			   (let ((vec (cons 'vec m))
+				 opos mm)
+			     (while (and (listp
+					  (setq opos exp-pos
+						mm (calc-match-user-syntax
+						    (or (nth 2 (car p))
+							(nth 1 (car p)))
+						    (car (nth 2 (car p))))))
+					 (> exp-pos opos))
+			       (setq vec (nconc vec mm)))
+			     (setq matches (nconc matches (list vec))))
+			 (and (eq (car (car p)) '*)
+			      (setq matches (nconc matches (list '(vec)))))))))
+      (setq p (cdr p)))
+    (if p
+	(setq exp-pos save-exp-pos
+	      exp-old-pos save-exp-old-pos
+	      exp-token save-exp-token
+	      exp-data save-exp-data
+	      matches "Failed"))
+    matches)
+)
+
 (defconst math-alg-inequalities
   '(calcFunc-lt calcFunc-gt calcFunc-leq calcFunc-geq
 		calcFunc-eq calcFunc-neq))
@@ -748,6 +988,9 @@
 		      (consp num))
 		 (list 'quote num)
 	       num)))
+	  ((and calc-user-parse-table
+		(setq op (calc-check-user-syntax)))
+	   op)
 	  ((or (equal exp-data "-")
 	       (equal exp-data "+")
 	       (equal exp-data "!")
@@ -783,16 +1026,27 @@
 				    (eq exp-token 'end)))
 			   (throw 'syntax "Expected `)'"))
 		       (math-read-token)
-		       (if f
-			   (setq sym (cdr f))
-			 (and (= (aref (symbol-name sym) 0) ?\\)
-			      (< (prefix-numeric-value calc-language-option) 0)
-			      (setq sym (intern (substring (symbol-name sym)
-							   1))))
-			 (or (string-match "-" (symbol-name sym))
-			     (setq sym (intern (concat "calcFunc-"
-						       (symbol-name sym))))))
-		       (cons sym args))))
+		       (if (and (eq calc-language 'fortran) args
+				(calc-extensions)
+				(let ((calc-matrix-mode 'scalar))
+				  (math-known-matrixp
+				   (list 'var sym
+					 (intern
+					  (concat "var-"
+						  (symbol-name sym)))))))
+			   (math-parse-fortran-subscr sym args)
+			 (if f
+			     (setq sym (cdr f))
+			   (and (= (aref (symbol-name sym) 0) ?\\)
+				(< (prefix-numeric-value calc-language-option)
+				   0)
+				(setq sym (intern (substring (symbol-name sym)
+							     1))))
+			   (or (string-match "-" (symbol-name sym))
+			       (setq sym (intern
+					  (concat "calcFunc-"
+						  (symbol-name sym))))))
+			 (cons sym args)))))
 	       (if math-read-expr-quotes
 		   sym
 		 (let ((val (list 'var
