@@ -192,7 +192,7 @@
   (setq name (calc-var-value name))
   (and (consp name)
        (memq (car name) '(vec calcFunc-assign calcFunc-condition))
-       (cdr name))
+       name)
 )
 
 (defun math-recompile-eval-rules ()
@@ -1218,6 +1218,20 @@
 )
 
 
+(math-defsimplify calcFunc-erf
+  (or (and (math-looks-negp (nth 1 expr))
+	   (math-neg (list 'calcFunc-erf (math-neg (nth 1 expr)))))
+      (and (eq (car-safe (nth 1 expr)) 'calcFunc-conj)
+	   (list 'calcFunc-conj (list 'calcFunc-erf (nth 1 (nth 1 expr))))))
+)
+
+(math-defsimplify calcFunc-erfc
+  (or (and (math-looks-negp (nth 1 expr))
+	   (math-sub 2 (list 'calcFunc-erfc (math-neg (nth 1 expr)))))
+      (and (eq (car-safe (nth 1 expr)) 'calcFunc-conj)
+	   (list 'calcFunc-conj (list 'calcFunc-erfc (nth 1 (nth 1 expr))))))
+)
+
 
 (defun math-linear-in (expr term &optional always)
   (if (math-expr-contains expr term)
