@@ -1386,7 +1386,7 @@
       (and (or (and (Math-vectorp a) (math-known-scalarp b))
 	       (and (Math-vectorp b) (math-known-scalarp a)))
 	   (math-map-vec-2 'math-mul a b))
-      (and (Math-objectp b)
+      (and (Math-objectp b) (not (Math-objectp a))
 	   (math-mul b a))
       (and (eq (car-safe a) 'neg)
 	   (math-neg (math-mul (nth 1 a) b)))
