@@ -58,6 +58,8 @@
       (let ((math-comp-selected nil))
 	(and math-comp-tagged (setq math-comp-tagged a))
 	(list 'tag a (math-compose-expr a prec))))
+     ((and (not (consp a)) (not (integerp a)))
+      (concat "'" (prin1-to-string a)))
      ((math-scalarp a)
       (if (or (eq (car-safe a) 'frac)
 	      (and (nth 1 calc-frac-format) (Math-integerp a)))
@@ -122,7 +124,6 @@
 					(math-match-substring s 2))
 				  (math-match-substring s 3))))
 		(if c (cons 'horiz (nconc c (list s))) s)))))))
-     ((not (consp a)) (concat "'" (prin1-to-string a)))
      ((and (get (car a) 'math-compose-forms)
 	   (not (eq calc-language 'unform))
 	   (let ((comps (get (car a) 'math-compose-forms))
@@ -149,7 +150,8 @@
 	     (row-commas (memq 'C calc-matrix-brackets))
 	     (comma-spc (or calc-vector-commas " "))
 	     (comma (or calc-vector-commas ""))
-	     (vector-prec (if (or (and calc-vector-commas (nth 2 a))
+	     (vector-prec (if (or (and calc-vector-commas
+				       (math-vector-no-parens a))
 				  (memq 'P calc-matrix-brackets)) 0 1000))
 	     (just (cond ((eq calc-matrix-just 'right) 'vright)
 			 ((eq calc-matrix-just 'center) 'vcent)
@@ -163,7 +165,8 @@
 		     (progn (setq break t) nil)))
 	    (if (progn
 		  (setq vector-prec (if (or (and calc-vector-commas
-						 (nth 2 (nth 1 a)))
+						 (math-vector-no-parens
+						  (nth 1 a)))
 					    (memq 'P calc-matrix-brackets))
 					0 1000))
 		  (= (length a) 2))
@@ -336,11 +339,16 @@
 	(concat "<" (math-format-date a) ">")))
      ((and (eq (car a) 'calcFunc-subscr) (cdr (cdr a))
 	   (memq calc-language '(c pascal fortran maple)))
-      (list 'horiz
-	    (math-compose-expr (nth 1 a) 1000)
-	    (if (eq calc-language 'fortran) "(" "[")
-	    (math-compose-vector (cdr (cdr a)) ", " 0)
-	    (if (eq calc-language 'fortran) ")" "]")))
+      (let ((args (cdr (cdr a))))
+	(while (and (memq calc-language '(pascal fortran))
+		    (eq (car-safe (nth 1 a)) 'calcFunc-subscr))
+	  (setq args (append (cdr (cdr (nth 1 a))) args)
+		a (nth 1 a)))
+	(list 'horiz
+	      (math-compose-expr (nth 1 a) 1000)
+	      (if (eq calc-language 'fortran) "(" "[")
+	      (math-compose-vector args ", " 0)
+	      (if (eq calc-language 'fortran) ")" "]"))))
      ((and (eq (car a) 'calcFunc-subscr) (= (length a) 3)
 	   (eq calc-language 'big))
       (let* ((a1 (math-compose-expr (nth 1 a) 1000))
@@ -825,6 +833,8 @@
 				  (symbol-name func))
 				 (math-match-substring (symbol-name func) 1)
 			       (symbol-name func))))
+		 (if (memq calc-language '(c fortran pascal maple))
+		     (setq func (math-to-underscores func)))
 		 (if (and (eq calc-language 'tex)
 			  calc-language-option
 			  (not (= calc-language-option 0))
@@ -911,29 +921,29 @@
     "")
 )
 
-(defun math-compose-matrix (a col cols base)
-  (math-compose-matrix-step a col)
+(defun math-vector-no-parens (a)
+  (or (cdr (cdr a))
+      (not (eq (car-safe (nth 1 a)) '*)))
 )
 
-(defun math-compose-matrix-step (a col)
-  (if (= col cols)
-      (list (cons just
-		  (cons base
-			(mapcar (function (lambda (r)
-					    (math-compose-expr
-					     (nth col r)
-					     vector-prec)))
-				a))))
-    (cons (cons just
-		(cons base
-		      (mapcar (function
-			       (lambda (r) (list 'horiz
-						 (math-compose-expr
-						  (nth col r)
-						  vector-prec)
-						 (concat comma-spc " "))))
-			      a)))
-	  (math-compose-matrix-step a (1+ col))))
+(defun math-compose-matrix (a col cols base)
+  (let ((col 0)
+	(res nil))
+    (while (<= (setq col (1+ col)) cols)
+      (setq res (cons (cons just
+			    (cons base
+				  (mapcar (function
+					   (lambda (r)
+					     (list 'horiz
+						   (math-compose-expr
+						    (nth col r)
+						    vector-prec)
+						   (if (= col cols)
+						       ""
+						     (concat comma-spc " ")))))
+					  a)))
+		      res)))
+    (nreverse res))
 )
 
 (defun math-compose-rows (a count first)
@@ -1354,7 +1364,7 @@
 		   (setq mrg (aref math-comp-buf-margin i))
 		   (if (> mrg 12)  ; indenting too far, go back to far left
 		       (let ((j i) (new (if calc-line-numbering 5 1)))
-			 (while (<= j comp-level)
+			 '(while (<= j comp-level)
 			   (aset math-comp-buf-margin j
 				 (+ (aref math-comp-buf-margin j) (- new mrg)))
 			   (setq j (1+ j)))
