@@ -139,12 +139,6 @@
    (calc-pop-push-record 1 "c%" (calc-to-percentage (calc-top-n 1))))
 )
 
-(defun calc-percent ()
-  (interactive)
-  (calc-slow-wrapper
-   (calc-pop-push-record 1 "%" (list 'calcFunc-percent (calc-top-n 1))))
-)
-
 (defun calc-percent-change ()
   (interactive)
   (calc-slow-wrapper
@@ -382,7 +376,7 @@
 				     '(var DUMMY var-DUMMY)
 				     flat)
 			       '(var DUMMY var-DUMMY)
-			       '(intv 3 '(float 1 -4) 1)
+			       '(intv 3 (float 1 -4) 1)
 			       t)))
     (if (math-vectorp root)
 	(nth 1 root)
