@@ -90,6 +90,8 @@
    (and arg
 	(>= (setq arg (prefix-numeric-value arg)) 0)
 	(<= arg 9)
+	(setq calc-standard-date-formats
+	      (copy-sequence calc-standard-date-formats))
 	(setcar (nthcdr arg calc-standard-date-formats) fmt))
    (let ((case-fold-search nil))
      (and (not (string-match "<.*>" fmt))
@@ -134,9 +136,9 @@
 	      (or (memq sym '(Y YY BY YYY YYYY
 				aa AA aaa AAA aaaa AAAA
 				bb BB bbb BBB bbbb BBBB
-				M MM BM Mmm Mmmm MMM MMMM
+				M MM BM mmm Mmm Mmmm MMM MMMM
 				D DD BD d ddd bdd
-				W Www Wwww WWW WWWW
+				W www Www Wwww WWW WWWW
 				h hh bh H HH BH
 				p P pp PP pppp PPPP
 				m mm bm s ss bss SS BS C
@@ -248,6 +250,18 @@
   (calc-date-one-arg "incm" 'calcFunc-incmonth arg)
 )
 
+(defun calc-business-days-plus (arg)
+  (interactive "P")
+  (calc-wrapper
+   (calc-binary-op "bus+" 'calcFunc-badd arg))
+)
+
+(defun calc-business-days-minus (arg)
+  (interactive "P")
+  (calc-wrapper
+   (calc-binary-op "bus-" 'calcFunc-bsub arg))
+)
+
 (defun calc-date-zero-args (prefix func arg)
   (calc-wrapper
    (if (consp arg)
@@ -634,6 +648,8 @@
 	 (format "%02d" month))
 	((eq x 'BM)
 	 (format "%2d" month))
+	((eq x 'mmm)
+	 (downcase (nth (1- month) math-short-month-names)))
 	((eq x 'Mmm)
 	 (nth (1- month) math-short-month-names))
 	((eq x 'MMM)
@@ -650,6 +666,8 @@
 	 (format "%2d" day))
 	((eq x 'W)
 	 (format "%d" weekday))
+	((eq x 'www)
+	 (downcase (nth weekday math-short-weekday-names)))
 	((eq x 'Www)
 	 (nth weekday math-short-weekday-names))
 	((eq x 'WWW)
@@ -847,7 +865,7 @@
 		(cond ((not (symbolp (car temp))))
 		      ((memq (car temp) '(Y YY BY YYY YYYY))
 		       (or year (setq year a a b b c)))
-		      ((memq (car temp) '(M MM BM Mmm Mmmm MMM MMMM))
+		      ((memq (car temp) '(M MM BM mmm Mmm Mmmm MMM MMMM))
 		       (or month (setq month a a b b c)))
 		      ((memq (car temp) '(D DD BD))
 		       (or day (setq day a a b b c))))
@@ -962,11 +980,11 @@
 			    year (car num)
 			    month (nth 1 num)
 			    day (nth 2 num))))
-		((memq this '(Mmm MMM))
+		((memq this '(mmm Mmm MMM))
 		 (setq month (math-parse-date-word math-short-month-names t)))
 		((memq this '(Mmmm MMMM))
 		 (setq month (math-parse-date-word math-long-month-names t)))
-		((memq this '(Www WWW))
+		((memq this '(www Www WWW))
 		 (math-parse-date-word math-short-weekday-names t))
 		((memq this '(Wwww WWWW))
 		 (math-parse-date-word math-long-weekday-names t))
@@ -1443,6 +1461,281 @@ and ends on the last Sunday of October at 2 a.m."
 
 
 
+(defun calcFunc-bsub (a b)
+  (or (eq (car-safe a) 'date)
+      (math-reject-arg a 'datep))
+  (if (eq (car-safe b) 'date)
+      (if (math-lessp (nth 1 a) (nth 1 b))
+	  (math-neg (calcFunc-bsub b a))
+	(math-setup-holidays b)
+	(let* ((da (math-to-business-day a))
+	       (db (math-to-business-day b)))
+	  (math-add (math-sub (car da) (car db))
+		    (if (and (cdr db) (not (cdr da))) 1 0))))
+    (calcFunc-badd a (math-neg b)))
+)
+
+(defun calcFunc-badd (a b)
+  (if (eq (car-safe b) 'date)
+      (if (eq (car-safe a) 'date)
+	  (math-reject-arg nil "*Illegal combination in date arithmetic")
+	(calcFunc-badd b a))
+    (if (eq (car-safe a) 'date)
+	(if (Math-realp b)
+	    (if (Math-zerop b)
+		a
+	      (let* ((d (math-to-business-day a))
+		     (bb (math-add (car d)
+				   (if (and (cdr d) (Math-posp b))
+				       (math-sub b 1) b))))
+		(or (math-from-business-day bb)
+		    (calcFunc-badd a b))))
+	  (if (eq (car-safe b) 'hms)
+	      (let ((hours (nth 7 math-holidays-cache)))
+		(setq b (math-div (math-from-hms b 'deg) 24))
+		(if hours
+		    (setq b (math-div b (cdr hours))))
+		(calcFunc-badd a b))
+	    (math-reject-arg nil "*Illegal combination in date arithmetic")))
+      (math-reject-arg a 'datep)))
+)
+
+(defun calcFunc-holiday (a)
+  (if (cdr (math-to-business-day a)) 1 0)
+)
+
+
+(setq math-holidays-cache nil)
+(setq math-holidays-cache-tag t)
+
+
+;;; Compute the number of business days since Jan 1, 1 AD.
+
+(defun math-to-business-day (date &optional need-year)
+  (if (eq (car-safe date) 'date)
+      (setq date (nth 1 date)))
+  (or (Math-realp date)
+      (math-reject-arg date 'datep))
+  (let* ((day (math-floor date))
+	 (time (math-sub date day))
+	 (dt (math-date-to-dt day))
+	 (delta 0)
+	 (holiday nil))
+    (or (not need-year) (eq (car dt) need-year)
+	(math-reject-arg (list 'date day) "*Generated holiday has wrong year"))
+    (math-setup-holidays date)
+    (let ((days (car math-holidays-cache)))
+      (while (and (setq days (cdr days)) (< (car days) day))
+	(setq delta (1+ delta)))
+      (and days (= day (car days))
+	   (setq holiday t)))
+    (let* ((weekdays (nth 3 math-holidays-cache))
+	   (weeks (1- (/ (+ day 6) 7)))
+	   (wkday (- day 1 (* weeks 7))))
+      (setq delta (+ delta (* weeks (length weekdays))))
+      (while (and weekdays (< (car weekdays) wkday))
+	(setq weekdays (cdr weekdays)
+	      delta (1+ delta)))
+      (and weekdays (eq wkday (car weekdays))
+	   (setq holiday t)))
+    (let ((hours (nth 7 math-holidays-cache)))
+      (if hours
+	  (progn
+	    (setq time (math-div (math-sub time (car hours)) (cdr hours)))
+	    (if (Math-lessp time 0) (setq time 0))
+	    (or (Math-lessp time 1)
+		(setq time
+		      (math-sub 1
+				(math-div 1 (math-mul 86400 (cdr hours)))))))))
+    (cons (math-add (math-sub day delta) time) holiday))
+)
+
+
+;;; Compute the date a certain number of business days since Jan 1, 1 AD.
+;;; If this returns NIL, holiday table was adjusted; redo calculation.
+
+(defun math-from-business-day (num)
+  (let* ((day (math-floor num))
+	 (time (math-sub num day)))
+    (or (integerp day)
+	(math-reject-arg nil "*Date is outside valid range"))
+    (math-setup-holidays)
+    (let ((days (nth 1 math-holidays-cache))
+	  (delta 0))
+      (while (and (setq days (cdr days)) (< (car days) day))
+	(setq delta (1+ delta)))
+      (setq day (+ day delta)))
+    (let* ((weekdays (nth 3 math-holidays-cache))
+	   (bweek (- 7 (length weekdays)))
+	   (weeks (1- (/ (+ day (1- bweek)) bweek)))
+	   (wkday (- day 1 (* weeks bweek)))
+	   (w 0))
+      (setq day (+ day (* weeks (length weekdays))))
+      (while (if (memq w weekdays)
+		 (setq day (1+ day))
+	       (> (setq wkday (1- wkday)) 0))
+	(setq w (1+ w)))
+      (let ((hours (nth 7 math-holidays-cache)))
+	(if hours
+	    (setq time (math-add (math-mul time (cdr hours)) (car hours)))))
+      (and (not (math-setup-holidays day))
+	   (list 'date (math-add day time)))))
+)
+
+
+(defun math-setup-holidays (&optional date)
+  (or (eq (calc-var-value 'var-Holidays) math-holidays-cache-tag)
+      (let ((h (calc-var-value 'var-Holidays))
+	    (wdnames '( (sun . 0) (mon . 1) (tue . 2) (wed . 3)
+			(thu . 4) (fri . 5) (sat . 6) ))
+	    (days nil) (weekdays nil) (exprs nil) (limit nil) (hours nil))
+	(or (math-vectorp h)
+	    (math-reject-arg h "*Holidays variable must be a vector"))
+	(while (setq h (cdr h))
+	  (cond ((or (and (eq (car-safe (car h)) 'date)
+			  (integerp (nth 1 (car h))))
+		     (and (eq (car-safe (car h)) 'intv)
+			  (eq (car-safe (nth 2 (car h))) 'date))
+		     (eq (car-safe (car h)) 'vec))
+		 (setq days (cons (car h) days)))
+		((and (eq (car-safe (car h)) 'var)
+		      (assq (nth 1 (car h)) wdnames))
+		 (setq weekdays (cons (cdr (assq (nth 1 (car h)) wdnames))
+				      weekdays)))
+		((and (eq (car-safe (car h)) 'intv)
+		      (eq (car-safe (nth 2 (car h))) 'hms)
+		      (eq (car-safe (nth 3 (car h))) 'hms))
+		 (if hours
+		     (math-reject-arg
+		      (car h) "*Only one hours interval allowed in Holidays"))
+		 (setq hours (math-div (car h) '(hms 24 0 0)))
+		 (if (or (Math-lessp (nth 2 hours) 0)
+			 (Math-lessp 1 (nth 3 hours)))
+		     (math-reject-arg
+		      (car h) "*Hours interval out of range"))
+		 (setq hours (cons (nth 2 hours)
+				   (math-sub (nth 3 hours) (nth 2 hours))))
+		 (if (Math-zerop (cdr hours))
+		     (math-reject-arg
+		      (car h) "*Degenerate hours interval")))
+		((or (and (eq (car-safe (car h)) 'intv)
+			  (Math-integerp (nth 2 (car h)))
+			  (Math-integerp (nth 3 (car h))))
+		     (and (integerp (car h))
+			  (> (car h) 1900) (< (car h) 2100)))
+		 (if limit
+		     (math-reject-arg
+		      (car h) "*Only one limit allowed in Holidays"))
+		 (setq limit (calcFunc-vint (car h) '(intv 3 1 2737)))
+		 (if (equal limit '(vec))
+		     (math-reject-arg (car h) "*Limit is out of range")))
+		((or (math-expr-contains (car h) '(var y var-y))
+		     (math-expr-contains (car h) '(var m var-m)))
+		 (setq exprs (cons (car h) exprs)))
+		(t (math-reject-arg
+		    (car h) "*Holidays must contain a vector of holidays"))))
+	(if (= (length weekdays) 7)
+	    (math-reject-arg nil "*Too many weekend days"))
+	(setq math-holidays-cache (list (list -1)  ; 0: days list
+					(list -1)  ; 1: inverse-days list
+					nil        ; 2: exprs
+					(sort weekdays '<)
+					(or limit '(intv 3 1 2737))
+					nil        ; 5: (lo.hi) expanded years
+					(cons exprs days)
+					hours)     ; 7: business hours
+	      math-holidays-cache-tag (calc-var-value 'var-Holidays))))
+  (if date
+      (let ((year (calcFunc-year date))
+	    (limits (nth 5 math-holidays-cache))
+	    (done nil))
+	(or (eq (calcFunc-in year (nth 4 math-holidays-cache)) 1)
+	    (progn
+	      (or (eq (car-safe date) 'date) (setq date (list 'date date)))
+	      (math-reject-arg date "*Date is outside valid range")))
+	(unwind-protect
+	    (let ((days (nth 6 math-holidays-cache)))
+	      (if days
+		  (let ((year nil))   ; see below
+		    (setcar (nthcdr 6 math-holidays-cache) nil)
+		    (math-setup-add-holidays (cons 'vec (cdr days)))
+		    (setcar (nthcdr 2 math-holidays-cache) (car days))))
+	      (cond ((not (nth 2 math-holidays-cache))
+		     (setq done t)
+		     nil)
+		    ((not limits)
+		     (setcar (nthcdr 5 math-holidays-cache) (cons year year))
+		     (math-setup-year-holidays year)
+		     (setq done t))
+		    ((< year (car limits))
+		     (message "Computing holidays, %d .. %d"
+			      year (1- (car limits)))
+		     (calc-set-command-flag 'clear-message)
+		     (while (< year (car limits))
+		       (setcar limits (1- (car limits)))
+		       (math-setup-year-holidays (car limits)))
+		     (setq done t))
+		    ((> year (cdr limits))
+		     (message "Computing holidays, %d .. %d"
+			      (1+ (cdr limits)) year)
+		     (calc-set-command-flag 'clear-message)
+		     (while (> year (cdr limits))
+		       (setcdr limits (1+ (cdr limits)))
+		       (math-setup-year-holidays (cdr limits)))
+		     (setq done t))
+		    (t
+		     (setq done t)
+		     nil)))
+	  (or done (setq math-holidays-cache-tag t)))))
+)
+
+(defun math-setup-year-holidays (year)
+  (let ((exprs (nth 2 math-holidays-cache)))
+    (while exprs
+      (let* ((var-y year)
+	     (var-m nil)
+	     (expr (math-evaluate-expr (car exprs))))
+	(if (math-expr-contains expr '(var m var-m))
+	    (let ((var-m 0))
+	      (while (<= (setq var-m (1+ var-m)) 12)
+		(math-setup-add-holidays (math-evaluate-expr expr))))
+	  (math-setup-add-holidays expr)))
+      (setq exprs (cdr exprs))))
+)
+
+(defun math-setup-add-holidays (days)   ; uses "year"
+  (cond ((eq (car-safe days) 'vec)
+	 (while (setq days (cdr days))
+	   (math-setup-add-holidays (car days))))
+	((eq (car-safe days) 'intv)
+	 (let ((day (math-ceiling (nth 2 days))))
+	   (or (eq (calcFunc-in day days) 1)
+	       (setq day (math-add day 1)))
+	   (while (eq (calcFunc-in day days) 1)
+	     (math-setup-add-holidays day)
+	     (setq day (math-add day 1)))))
+	((eq (car-safe days) 'date)
+	 (math-setup-add-holidays (nth 1 days)))
+	((eq days 0))
+	((integerp days)
+	 (let ((b (math-to-business-day days year)))
+	   (or (cdr b)   ; don't register holidays twice!
+	       (let ((prev (car math-holidays-cache))
+		     (iprev (nth 1 math-holidays-cache)))
+		 (while (and (cdr prev) (< (nth 1 prev) days))
+		   (setq prev (cdr prev) iprev (cdr iprev)))
+		 (setcdr prev (cons days (cdr prev)))
+		 (setcdr iprev (cons (car b) (cdr iprev)))
+		 (while (setq iprev (cdr iprev))
+		   (setcar iprev (1- (car iprev))))))))
+	((Math-realp days)
+	 (math-reject-arg (list 'date days) "*Invalid holiday value"))
+	(t
+	 (math-reject-arg days "*Holiday formula failed to evaluate")))
+)
+
+
+
 
 ;;;; Error forms.
 
