@@ -346,12 +346,14 @@
       '(float 1 0)
     (if (equal x '(neg (var inf var-inf)))
 	'(float -1 0)
-      (let ((math-current-gamma-value (math-sqrt-pi)))
-	(math-to-same-complex-quad
-	 (math-div (calcFunc-gammag '(float 5 -1)
-				    (math-sqr (math-to-complex-quad-one x)))
-		   math-current-gamma-value)
-	 x))))
+      (if (Math-zerop x)
+	  x
+	(let ((math-current-gamma-value (math-sqrt-pi)))
+	  (math-to-same-complex-quad
+	   (math-div (calcFunc-gammag '(float 5 -1)
+				      (math-sqr (math-to-complex-quad-one x)))
+		     math-current-gamma-value)
+	   x)))))
 )
 
 (defun calcFunc-erfc (x)
@@ -361,7 +363,7 @@
 	(let ((math-current-gamma-value (math-sqrt-pi)))
 	  (math-div (calcFunc-gammaG '(float 5 -1) (math-sqr x))
 		    math-current-gamma-value))
-      (math-add '(float 1 0) (calcFunc-erf (math-neg x)))))
+      (math-sub 1 (calcFunc-erf x))))
 )
 
 (defun math-to-complex-quad-one (x)
