@@ -1225,14 +1225,21 @@ This \"dumb\" driver will be present in Gnuplot 3.0."
 )
 
 (defun calc-graph-output (name flag)
-  (interactive "sOutput file name: \nP")
+  (interactive "FOutput file name: \np")
+  (cond ((string-match "\\<[aA][uU][tT][oO]$" name)
+	 (setq name "auto"))
+	((string-match "\\<[tT][tT][yY]$" name)
+	 (setq name "tty"))
+	((string-match "\\<[sS][tT][dD][oO][uU][tT]$" name)
+	 (setq name "STDOUT"))
+	((equal (file-name-nondirectory name) "")
+	 (setq name ""))
+	(t (setq name (expand-file-name name))))
   (if flag
       (if (> (prefix-numeric-value flag) 0)
 	  (if (equal name "")
 	      (message "Default GNUPLOT output file is \"%s\"."
 		       calc-gnuplot-default-output)
-	    (if (string-match "^[sS][tT][dD][oO][uU][tT]$" name)
-		(setq name "STDOUT"))
 	    (setq calc-gnuplot-default-output name))
 	(if (equal name "")
 	    (message "GNUPLOT output file for Print command is \"%s\"."
