@@ -1,6 +1,6 @@
 ;; Calculator for GNU Emacs, part II [calc-keypd.el]
-;; Copyright (C) 1990, 1991 Free Software Foundation, Inc.
-;; Written by Dave Gillespie, daveg@csvax.cs.caltech.edu.
+;; Copyright (C) 1990, 1991, 1992, 1993 Free Software Foundation, Inc.
+;; Written by Dave Gillespie, daveg@synaptics.com.
 
 ;; This file is part of GNU Emacs.
 
@@ -52,6 +52,8 @@
   (define-key calc-keypad-map "q" 'calc-keypad-off))
 
 (defun calc-do-keypad (&optional full-display interactive)
+  (if (string-match "^19" emacs-version)
+      (error "Sorry, calc-keypad not yet implemented for Emacs 19"))
   (calc-create-buffer)
   (let ((calcbuf (current-buffer)))
     (or (and calc-keypad-buffer
@@ -603,10 +605,7 @@
        ( "HEX"   calc-hex-radix )
        ( "OCT"   calc-octal-radix )
        ( "BIN"   calc-binary-radix )
-       ( "WSIZ"  (progn
-		   (lambda (arg) (interactive "nWord size: ")
-		     (setq prefix-arg arg))
-		   calc-word-size) calc-word-size )
+       ( "WSIZ"  calc-word-size )
        ( "ARSH"  calc-rshift-arith ) )
      ( ( "A"     ("A") )
        ( "B"     ("B") )
@@ -626,7 +625,7 @@
 (defvar calc-keypad-vector-menu
   '( ( ( "SUM"   calc-vector-sum calc-vector-alt-sum calc-vector-mean )
        ( "PROD"  calc-vector-product nil calc-vector-sdev )
-       ( "MAX"   calc-vector-max calc-vector-min )
+       ( "MAX"   calc-vector-max calc-vector-min calc-vector-median )
        ( "MAP*"  (lambda () (interactive)
 		   (calc-map '(2 calcFunc-mul "*"))) )
        ( "MAP^"  (lambda () (interactive)
@@ -657,10 +656,14 @@
 ;;; |SWAP|RLL3|RLL4|OVER|STO |RCL |
 
 (defvar calc-keypad-modes-menu
-  '( ( ( "FLT"   calc-normal-notation )
-       ( "FIX"   calc-fix-notation )
-       ( "SCI"   calc-sci-notation )
-       ( "ENG"   calc-eng-notation )
+  '( ( ( "FLT"   calc-normal-notation
+	         (progn calc-num-prefix calc-normal-notation) )
+       ( "FIX"   (progn 2 calc-fix-notation)
+	         (progn calc-num-prefix calc-fix-notation) )
+       ( "SCI"   calc-sci-notation
+	         (progn calc-num-prefix calc-sci-notation) )
+       ( "ENG"   calc-eng-notation
+	         (progn calc-num-prefix calc-eng-notation) )
        ( "GRP"   calc-group-digits "\C-u-3\excalc-group-digits\r" )
        ( ""	 nil ) )
      ( ( "RAD"   calc-radians-mode )
