@@ -183,7 +183,17 @@ Unix usage:
 
 
 
-(defun calc-split-manual (&optional force)
+(defun calc-split-tutorial (&optional force)
+  (interactive "P")
+  (calc-split-manual force 1))
+
+
+(defun calc-split-reference (&optional force)
+  (interactive "P")
+  (calc-split-manual force 2))
+
+
+(defun calc-split-manual (&optional force part)
   "Split the Calc manual into separate Tutorial and Reference manuals.
 Use this if your TeX installation is too small-minded to handle
 calc.texinfo all at once.
@@ -206,22 +216,28 @@ Usage:  C-x C-f calc.texinfo RET
     (search-forward "@c [end]")
     (beginning-of-line)
     (setq endpos (point))
-    (find-file "calctut.tex")
-    (erase-buffer)
-    (insert-buffer-substring srcbuf 1 refpos)
-    (insert-buffer-substring srcbuf endpos maxpos)
-    (calc-split-volume "I" "ref" "Tutorial" "Reference")
-    (save-buffer)
-    (find-file "calcref.tex")
-    (erase-buffer)
-    (insert-buffer-substring srcbuf 1 tutpos)
-    (insert "\n@tex\n\\global\\advance\\chapno by 1\n@end tex\n")
-    (insert-buffer-substring srcbuf refpos maxpos)
-    (calc-split-volume "II" "tut" "Reference" "Tutorial")
-    (save-buffer)
+    (or (eq part 2)
+	(progn
+	  (find-file "calctut.tex")
+	  (erase-buffer)
+	  (insert-buffer-substring srcbuf 1 refpos)
+	  (insert-buffer-substring srcbuf endpos maxpos)
+	  (calc-split-volume "I" "ref" "Tutorial" "Reference")
+	  (save-buffer)))
+    (or (eq part 1)
+	(progn
+	  (find-file "calcref.tex")
+	  (erase-buffer)
+	  (insert-buffer-substring srcbuf 1 tutpos)
+	  (insert "\n@tex\n\\global\\advance\\chapno by 1\n@end tex\n")
+	  (insert-buffer-substring srcbuf refpos maxpos)
+	  (calc-split-volume "II" "tut" "Reference" "Tutorial")
+	  (save-buffer)))
     (switch-to-buffer srcbuf)
     (goto-char 1))
-  (message "Wrote files calctut.tex and calcref.tex")
+  (message (cond ((eq part 1) "Wrote file calctut.tex")
+		 ((eq part 2) "Wrote file calcref.tex")
+		 (t "Wrote files calctut.tex and calcref.tex")))
 )
 
 (defun calc-split-volume (number fix name other-name)
@@ -254,7 +270,12 @@ Usage:  C-x C-f calc.texinfo RET
 )
 
 
-(defun calc-split-summary (&optional force)
+(defun calc-inline-summary ()
+  "Make a special \"calcsum.tex\" file to be used with main manual."
+  (calc-split-summary nil t)
+)
+
+(defun calc-split-summary (&optional force in-line)
   "Make a special \"calcsum.tex\" file with just the Calc summary."
   (interactive "P")
   (or (let ((case-fold-search t))
@@ -262,7 +283,7 @@ Usage:  C-x C-f calc.texinfo RET
       force
       (error "This command should be used in the calc.texinfo buffer."))
   (let ((srcbuf (current-buffer))
-	begpos sumpos endpos)
+	begpos sumpos endpos midpos)
     (goto-char 1)
     (search-forward "{Calc Manual}")
     (backward-char 1)
@@ -282,16 +303,90 @@ Usage:  C-x C-f calc.texinfo RET
     (insert-buffer-substring srcbuf 1 begpos)
     (insert "@tex\n"
 	    "\\global\\advance\\appendixno2\n"
-	    "\\gdef\\xref#1.{See ``#1.''}\n"
-	    "@end tex\n")
+	    "\\gdef\\xref#1.{See ``#1.''}\n")
+    (setq midpos (point))
+    (insert "@end tex\n")
     (insert-buffer-substring srcbuf sumpos endpos)
     (insert "@bye\n")
     (goto-char 1)
-    (if (search-forward "@c smallbook" nil t)
-	(progn   ; activate "smallbook" format for compactness
-	  (beginning-of-line)
-	  (forward-char 1)
-	  (delete-char 2)))
+    (if (search-forward "{. a b c" nil t)
+	(replace-match "{... a b c"))
+    (goto-char 1)
+    (if in-line
+	(let ((buf (current-buffer))
+	      (page nil))
+	  (find-file "calc.aux")
+	  (if (> (buffer-size) 0)
+	      (progn
+		(goto-char 1)
+		(re-search-forward "{Summary-pg}{\\([0-9]+\\)}")
+		(setq page (string-to-int (buffer-substring (match-beginning 1)
+							    (match-end 1))))))
+	  (switch-to-buffer buf)
+	  (if page
+	      (progn
+		(message "Adjusting starting page number to %d" page)
+		(goto-char midpos)
+		(insert (format "\\global\\pageno=%d\n" page)))
+	    (message "Unable to find page number from calc.aux")))
+      (if (search-forward "@c smallbook" nil t)
+	  (progn   ; activate "smallbook" format for compactness
+	    (beginning-of-line)
+	    (forward-char 1)
+	    (delete-char 2))))
+    (let ((buf (current-buffer)))
+      (find-file "calc.ky")
+      (if (> (buffer-size) 0)
+	  (let ((ibuf (current-buffer)))
+	    (message "Mixing in page numbers from Key Index (calc.ky)")
+	    (switch-to-buffer buf)
+	    (goto-char 1)
+	    (search-forward "notes at the end")
+	    (insert "; the number in italics is\n"
+		    "the page number where the command is described")
+	    (while (re-search-forward
+		    "@r{.*@: *\\([^ ]\\(.*[^ ]\\)?\\) *@:.*@:.*@:\\(.*\\)@:.*}"
+		    nil t)
+	      (let ((key (buffer-substring (match-beginning 1) (match-end 1)))
+		    (pos (match-beginning 3))
+		    num)
+		(set-buffer ibuf)
+		(goto-char 1)
+		(let ((p '( ( "I H " . "H I " )  ; oops!
+			    ( "@@ ' \"" . "@@" ) ( "h m s" . "@@" )
+			    ( "\\\\" . "{\\tt\\indexbackslash }" )
+			    ( "_" . "{\\_}" )
+			    ( "\\^" . "{\\tt\\hat}" )
+			    ( "<" . "{\\tt\\less}" )
+			    ( ">" . "{\\tt\\gtr}" )
+			    ( "\"" ) ( "@{" ) ( "@}" )
+			    ( "~" ) ( "|" ) ( "@@" )
+			    ( "\\+" . "{\\tt\\char43}" )
+			    ( "# l" . "# L" )
+			    ( "I f I" . "f I" ) ( "I f Q" . "f Q" )
+			    ( "V &" . "&" ) ( "C-u " . "" ) ))
+		      (case-fold-search nil))
+		  (while p
+		    (if (string-match (car (car p)) key)
+			(setq key (concat (substring key 0 (match-beginning 0))
+					  (or (cdr (car p))
+					      (format "{\\tt\\char'%03o}"
+						      (aref key (1- (match-end
+								     0)))))
+					  (substring key (match-end 0)))))
+		    (setq p (cdr p)))
+		  (setq num (and (search-forward (format "\\entry {%s}{" key)
+						 nil t)
+				 (looking-at "[0-9]+")
+				 (buffer-substring (point) (match-end 0)))))
+		(set-buffer buf)
+		(goto-char pos)
+		(insert "@pgref{" (or num "") "}")))
+	    (goto-char midpos)
+	    (insert "\\gdef\\pgref#1{\\hbox to 2em{\\indsl\\hss#1}\\ \\ }\n"))
+	(message
+	 "Unable to find Key Index (calc.ky); no page numbers inserted"))
+      (switch-to-buffer buf))
     (save-buffer))
   (message "Wrote file calcsum.tex")
 )
