@@ -1,6 +1,6 @@
 ;; Calculator for GNU Emacs, part II [calc-map.el]
-;; Copyright (C) 1990, 1991 Free Software Foundation, Inc.
-;; Written by Dave Gillespie, daveg@csvax.cs.caltech.edu.
+;; Copyright (C) 1990, 1991, 1992, 1993 Free Software Foundation, Inc.
+;; Written by Dave Gillespie, daveg@synaptics.com.
 
 ;; This file is part of GNU Emacs.
 
@@ -1013,7 +1013,10 @@
 	  (setq row (car vec))
 	  (while (setq row (cdr row))
 	    (setq expr (if expr
-			   (math-build-call func (list expr (car row)))
+			   (if (Math-numberp expr)
+			       (math-normalize
+				(math-build-call func (list expr (car row))))
+			     (math-build-call func (list expr (car row))))
 			 (car row)))))
 	(math-normalize expr))
     (calcFunc-reducer func vec))
@@ -1043,6 +1046,22 @@
   (let ((expr (car (setq vec (cdr vec)))))
     (if expr
 	(progn
+	  (condition-case err
+	      (and (symbolp func)
+		   (let ((lfunc (or (cdr (assq func
+					       '( (calcFunc-add . math-add)
+						  (calcFunc-sub . math-sub)
+						  (calcFunc-mul . math-mul)
+						  (calcFunc-div . math-div)
+						  (calcFunc-pow . math-pow)
+						  (calcFunc-mod . math-mod)
+						  (calcFunc-vconcat .
+						   math-concat) )))
+				    lfunc)))
+		     (while (cdr vec)
+		       (setq expr (funcall lfunc expr (nth 1 vec))
+			     vec (cdr vec)))))
+	    (error nil))
 	  (while (setq vec (cdr vec))
 	    (setq expr (math-build-call func (list expr (car vec)))))
 	  (math-normalize expr))
