@@ -260,11 +260,15 @@
 )
 
 
-(defun calc-degrees-mode ()
-  (interactive)
-  (calc-wrapper
-   (calc-change-mode 'calc-angle-mode 'deg)
-   (message "Angles measured in degrees."))
+(defun calc-degrees-mode (arg)
+  (interactive "p")
+  (cond ((= arg 1)
+	 (calc-wrapper
+	  (calc-change-mode 'calc-angle-mode 'deg)
+	  (message "Angles measured in degrees.")))
+	((= arg 2) (calc-radians-mode))
+	((= arg 3) (calc-hms-mode))
+	(t (error "Prefix argument out of range")))
 )
 
 (defun calc-radians-mode ()
