@@ -47,9 +47,9 @@ For turning Calc on and off:
 For moving data into and out of Calc:
   G  calc-grab-region.  Grab the region defined by mark and point into Calc.
   R  calc-grab-rectangle.  Grab the rectangle defined by mark, point into Calc.
-  Y  calc-copy-to-buffer.  Copy a value from the stack into the editing buffer.
   :  calc-grab-sum-down.  Grab a rectangle and sum the columns.
   _  calc-grab-sum-across.  Grab a rectangle and sum the rows.
+  Y  calc-copy-to-buffer.  Copy a value from the stack into the editing buffer.
 
 For use with Embedded mode:
   A  calc-embedded-activate.  Find and activate all :='s and =>'s in buffer.
@@ -235,7 +235,9 @@ Calc user interface as before (either M-# C or M-# K; initially M-# C)."
 							  (car p))))))
 	   (setq p (cdr p)))
 	 (if p (setq calc-info-filename name))))
-  (info)
+  (condition-case err
+      (info)
+    (error nil))
   (or (and (boundp 'Info-current-file)
 	   (stringp Info-current-file)
 	   (string-match "calc" Info-current-file))
@@ -262,7 +264,7 @@ Calc user interface as before (either M-# C or M-# K; initially M-# C)."
 
 (defun calc-help ()
   (interactive)
-  (let ((msgs
+  (let ((msgs (append
 	 '("Press `h' for complete help; press `?' repeatedly for a summary"
 	   "Letter keys: Negate; Precision; Yank; Why; Xtended cmd; Quit"
 	   "Letter keys: SHIFT + Undo, reDo; Keep-args; Inverse, Hyperbolic"
@@ -281,8 +283,10 @@ Calc user interface as before (either M-# C or M-# K; initially M-# C)."
 	   "Prefix keys: Kombinatorics/statistics, Modes, Store/recall"
 	   "Prefix keys: Trail/time, Units/statistics, Vector/matrix"
 	   "Prefix keys: Z (user), SHIFT + Z (define)"
-	   "Prefix keys: prefix + ? gives further help for that prefix"
-	   "  Calc 2.01 by Dave Gillespie, daveg@csvax.cs.caltech.edu")))
+	   "Prefix keys: prefix + ? gives further help for that prefix")
+	 (list (format
+		"  Calc %s by Dave Gillespie, daveg@csvax.cs.caltech.edu"
+		calc-version)))))
     (if calc-full-help-flag
 	msgs
       (if (or calc-inverse-flag calc-hyperbolic-flag)
@@ -441,6 +445,13 @@ Calc user interface as before (either M-# C or M-# K; initially M-# C)."
    (calc-unary-op "inv" 'calcFunc-inv arg))
 )
 
+(defun calc-percent ()
+  (interactive)
+  (calc-slow-wrapper
+   (calc-pop-push-record-list
+    1 "%" (list (list 'calcFunc-percent (calc-top-n 1)))))
+)
+
 
 (defun calc-over (n)
   (interactive "P")
