@@ -166,7 +166,12 @@
   (interactive "P")
   (calc-wrapper
    (calc-change-mode 'calc-float-format
-		     (setq n (list 'float (if n (prefix-numeric-value n) 0)))
+		     (let* ((val (if n (prefix-numeric-value n) 0))
+			    (mode (/ (+ val 5000) 10000)))
+		       (if (or (< val -5000) (> mode 3))
+			   (error "Prefix out of range"))
+		       (setq n (list (aref [float sci eng fix] mode)
+				     (- (% (+ val 5000) 10000) 5000))))
 		     t)
    (if (eq (nth 1 n) 0)
        (message "Displaying floating-point numbers normally.")
@@ -339,6 +344,53 @@
 	 (message "New file"))))
 )
 
+(defun math-get-modes-vec ()
+  (list 'vec
+	calc-internal-prec
+	calc-word-size
+	(calc-stack-size)
+	calc-number-radix
+	(+ (if (<= (nth 1 calc-float-format) 0)
+	       (+ calc-internal-prec (nth 1 calc-float-format))
+	     (nth 1 calc-float-format))
+	   (cdr (assq (car calc-float-format)
+		      '((float . 0) (sci . 10000)
+			(eng . 20000) (fix . 30000)))))
+	(cond ((eq calc-angle-mode 'rad) 2)
+	      ((eq calc-angle-mode 'hms) 3)
+	      (t 1))
+	(if calc-symbolic-mode 1 0)
+	(if calc-prefer-frac 1 0)
+	(if (eq calc-complex-mode 'polar) 1 0)
+	(cond ((eq calc-matrix-mode 'scalar) 0)
+	      ((eq calc-matrix-mode 'matrix) -2)
+	      (calc-matrix-mode)
+	      (t -1))
+	(cond ((eq calc-simplify-mode 'none) -1)
+	      ((eq calc-simplify-mode 'num) 0)
+	      ((eq calc-simplify-mode 'binary) 2)
+	      ((eq calc-simplify-mode 'alg) 3)
+	      ((eq calc-simplify-mode 'ext) 4)
+	      ((eq calc-simplify-mode 'units) 5)
+	      (t 1))
+	(cond ((eq calc-infinite-mode 1) 0)
+	      (calc-infinite-mode 1)
+	      (t -1)))
+)
+
+(defun calc-get-modes (n)
+  (interactive "P")
+  (calc-wrapper
+   (let ((modes (math-get-modes-vec)))
+     (calc-enter-result 0 "mode"
+			(if n
+			    (if (and (>= (setq n (prefix-numeric-value n)) 1)
+				     (< n (length modes)))
+				(nth n modes)
+			      (error "Prefix out of range"))
+			  modes))))
+)
+
 (defun calc-shift-prefix (arg)
   (interactive "P")
   (calc-wrapper
@@ -442,7 +494,7 @@
    (calc-change-mode 'calc-matrix-mode
 		     (cond ((eq arg 0) 'scalar)
 			   ((< (prefix-numeric-value arg) 1)
-			    (error "Dimension must be 1 or more"))
+			    (and (< (prefix-numeric-value arg) -1) 'matrix))
 			   (arg (prefix-numeric-value arg))
 			   ((eq calc-matrix-mode 'matrix) 'scalar)
 			   ((eq calc-matrix-mode 'scalar) nil)
@@ -483,10 +535,19 @@
 			   "Default simplifications apply only if arguments are numeric."))
 )
 
-(defun calc-default-simplify-mode ()
-  (interactive)
-  (calc-wrapper
-   (calc-set-simplify-mode nil nil "Usual default simplifications are enabled."))
+(defun calc-default-simplify-mode (arg)
+  (interactive "p")
+  (cond ((= arg 1)
+	 (calc-wrapper
+	  (calc-set-simplify-mode
+	   nil nil "Usual default simplifications are enabled.")))
+	((= arg 0) (calc-num-simplify-mode 1))
+	((< arg 0) (calc-no-simplify-mode 1))
+	((= arg 2) (calc-bin-simplify-mode 1))
+	((= arg 3) (calc-alg-simplify-mode 1))
+	((= arg 4) (calc-ext-simplify-mode 1))
+	((= arg 5) (calc-units-simplify-mode 1))
+	(t (error "Prefix argument out of range")))
 )
 
 (defun calc-bin-simplify-mode (arg)
