@@ -847,8 +847,9 @@
 (defun calcFunc-nrat (expr)
   (if (math-any-floats expr)
       (setq expr (calcFunc-pfrac expr)))
-  (if (math-vectorp expr)
-      (cons 'vec (mapcar 'calcFunc-nrat (cdr expr)))
+  (if (or (math-vectorp expr)
+	  (assq (car-safe expr) calc-tweak-eqn-table))
+      (cons (car expr) (mapcar 'calcFunc-nrat (cdr expr)))
     (let* ((calc-prefer-frac t)
 	   (res (math-to-ratpoly expr))
 	   (num (math-simplify (math-sort-terms (calcFunc-expand (car res)))))
