@@ -1,6 +1,6 @@
 ;; Calculator for GNU Emacs, part II [calc-prog.el]
-;; Copyright (C) 1990, 1991 Free Software Foundation, Inc.
-;; Written by Dave Gillespie, daveg@csvax.cs.caltech.edu.
+;; Copyright (C) 1990, 1991, 1992, 1993 Free Software Foundation, Inc.
+;; Written by Dave Gillespie, daveg@synaptics.com.
 
 ;; This file is part of GNU Emacs.
 
@@ -447,6 +447,197 @@
 )
 
 
+(defun calc-edit-user-syntax ()
+  (interactive)
+  (calc-wrapper
+   (let ((lang calc-language))
+     (calc-edit-mode (list 'calc-finish-user-syntax-edit (list 'quote lang))
+		     t
+		     (format "Editing %s-Mode Syntax Table"
+			     (cond ((null lang) "Normal")
+				   ((eq lang 'tex) "TeX")
+				   (t (capitalize (symbol-name lang))))))
+     (calc-write-parse-table (cdr (assq lang calc-user-parse-tables))
+			     lang)))
+  (calc-show-edit-buffer)
+)
+
+(defun calc-finish-user-syntax-edit (lang)
+  (let ((tab (calc-read-parse-table calc-original-buffer lang))
+	(entry (assq lang calc-user-parse-tables)))
+    (if tab
+	(setcdr (or entry
+		    (car (setq calc-user-parse-tables
+			       (cons (list lang) calc-user-parse-tables))))
+		tab)
+      (if entry
+	  (setq calc-user-parse-tables
+		(delq entry calc-user-parse-tables)))))
+  (switch-to-buffer calc-original-buffer)
+)
+
+(defun calc-write-parse-table (tab calc-lang)
+  (let ((p tab))
+    (while p
+      (calc-write-parse-table-part (car (car p)))
+      (insert ":= "
+	      (let ((math-format-hash-args t))
+		(math-format-flat-expr (cdr (car p)) 0))
+	      "\n")
+      (setq p (cdr p))))
+)
+
+(defun calc-write-parse-table-part (p)
+  (while p
+    (cond ((stringp (car p))
+	   (let ((s (car p)))
+	     (if (and (string-match "\\`\\\\dots\\>" s)
+		      (not (eq calc-lang 'tex)))
+		 (setq s (concat ".." (substring s 5))))
+	     (if (or (and (string-match
+			   "[a-zA-Z0-9\"{}]\\|\\`:=\\'\\|\\`#\\|\\`%%" s)
+			  (string-match "[^a-zA-Z0-9\\]" s))
+		     (and (assoc s '((")") ("]") (">")))
+			  (not (cdr p))))
+		 (insert (prin1-to-string s) " ")
+	       (insert s " "))))
+	  ((integerp (car p))
+	   (insert "#")
+	   (or (= (car p) 0)
+	       (insert "/" (int-to-string (car p))))
+	   (insert " "))
+	  ((and (eq (car (car p)) '\?) (equal (car (nth 2 (car p))) "$$"))
+	   (insert (car (nth 1 (car p))) " "))
+	  (t
+	   (insert "{ ")
+	   (calc-write-parse-table-part (nth 1 (car p)))
+	   (insert "}" (symbol-name (car (car p))))
+	   (if (nth 2 (car p))
+	       (calc-write-parse-table-part (list (car (nth 2 (car p)))))
+	     (insert " "))))
+    (setq p (cdr p)))
+)
+
+(defun calc-read-parse-table (calc-buf calc-lang)
+  (let ((tab nil))
+    (while (progn
+	     (skip-chars-forward "\n\t ")
+	     (not (eobp)))
+      (if (looking-at "%%")
+	  (end-of-line)
+	(let ((pt (point))
+	      (p (calc-read-parse-table-part ":=[\n\t ]+" ":=")))
+	  (or (stringp (car p))
+	      (and (integerp (car p))
+		   (stringp (nth 1 p)))
+	      (progn
+		(goto-char pt)
+		(error "Malformed syntax rule")))
+	  (let ((pos (point)))
+	    (end-of-line)
+	    (let* ((str (buffer-substring pos (point)))
+		   (exp (save-excursion
+			  (set-buffer calc-buf)
+			  (let ((calc-user-parse-tables nil)
+				(calc-language nil)
+				(math-expr-opers math-standard-opers)
+				(calc-hashes-used 0))
+			    (math-read-expr
+			     (if (string-match ",[ \t]*\\'" str)
+				 (substring str 0 (match-beginning 0))
+			       str))))))
+	      (if (eq (car-safe exp) 'error)
+		  (progn
+		    (goto-char (+ pos (nth 1 exp)))
+		    (error (nth 2 exp))))
+	      (setq tab (nconc tab (list (cons p exp)))))))))
+    tab)
+)
+
+(defun calc-fix-token-name (name &optional unquoted)
+  (cond ((string-match "\\`\\.\\." name)
+	 (concat "\\dots" (substring name 2)))
+	((and (equal name "{") (memq calc-lang '(tex eqn)))
+	 "(")
+	((and (equal name "}") (memq calc-lang '(tex eqn)))
+	 ")")
+	((and (equal name "&") (eq calc-lang 'tex))
+	 ",")
+	((equal name "#")
+	 (search-backward "#")
+	 (error "Token '#' is reserved"))
+	((and unquoted (string-match "#" name))
+	 (error "Tokens containing '#' must be quoted"))
+	((not (string-match "[^ ]" name))
+	 (search-backward "\"" nil t)
+	 (error "Blank tokens are not allowed"))
+	(t name))
+)
+
+(defun calc-read-parse-table-part (term eterm)
+  (let ((part nil)
+	(quoted nil))
+    (while (progn
+	     (skip-chars-forward "\n\t ")
+	     (if (eobp) (error "Expected '%s'" eterm))
+	     (not (looking-at term)))
+      (cond ((looking-at "%%")
+	     (end-of-line))
+	    ((looking-at "{[\n\t ]")
+	     (forward-char 2)
+	     (let ((p (calc-read-parse-table-part "}" "}")))
+	       (or (looking-at "[+*?]")
+		   (error "Expected '+', '*', or '?'"))
+	       (let ((sym (intern (buffer-substring (point) (1+ (point))))))
+		 (forward-char 1)
+		 (looking-at "[^\n\t ]*")
+		 (let ((sep (buffer-substring (point) (match-end 0))))
+		   (goto-char (match-end 0))
+		   (and (eq sym '\?) (> (length sep) 0)
+			(not (equal sep "$")) (not (equal sep "."))
+			(error "Separator not allowed with { ... }?"))
+		   (if (string-match "\\`\"" sep)
+		       (setq sep (read-from-string sep)))
+		   (setq sep (calc-fix-token-name sep))
+		   (setq part (nconc part
+				     (list (list sym p
+						 (and (> (length sep) 0)
+						      (cons sep p))))))))))
+	    ((looking-at "}")
+	     (error "Too many }'s"))
+	    ((looking-at "\"")
+	     (setq quoted (calc-fix-token-name (read (current-buffer)))
+		   part (nconc part (list quoted))))
+	    ((looking-at "#\\(\\(/[0-9]+\\)?\\)[\n\t ]")
+	     (setq part (nconc part (list (if (= (match-beginning 1)
+						 (match-end 1))
+					      0
+					    (string-to-int
+					     (buffer-substring
+					      (1+ (match-beginning 1))
+					      (match-end 1)))))))
+	     (goto-char (match-end 0)))
+	    ((looking-at ":=[\n\t ]")
+	     (error "Misplaced ':='"))
+	    (t
+	     (looking-at "[^\n\t ]*")
+	     (let ((end (match-end 0)))
+	       (setq part (nconc part (list (calc-fix-token-name
+					     (buffer-substring
+					      (point) end) t))))
+	       (goto-char end)))))
+    (goto-char (match-end 0))
+    (let ((len (length part)))
+      (while (and (> len 1)
+		  (let ((last (nthcdr (setq len (1- len)) part)))
+		    (and (assoc (car last) '((")") ("]") (">")))
+			 (not (eq (car last) quoted))
+			 (setcar last
+				 (list '\? (list (car last)) '("$$"))))))))
+    part)
+)
+
+
 (defun calc-user-define-invocation ()
   (interactive)
   (or last-kbd-macro
@@ -888,7 +1079,7 @@
 )
 
 (defun calc-execute-kbd-macro (mac arg &rest prefix)
-  (if (vectorp mac)
+  (if (and (vectorp mac) (> (length mac) 0) (stringp (aref mac 0)))
       (setq mac (or (aref mac 1)
 		    (aset mac 1 (progn (and (fboundp 'edit-kbd-macro)
 					    (edit-kbd-macro nil))
@@ -1883,14 +2074,14 @@
 				 (if (and (math-numberp (nth 1 args))
 					  (math-zerop (nth 1 args)))
 				     (list 'math-posp (car args))
-				   (cons 'math-lessp args))))
+				   (list 'math-lessp
+					 (nth 1 args) (nth 0 args)))))
 			  ((eq func '>=)
 			   (list 'not
 				 (if (and (math-numberp (nth 1 args))
 					  (math-zerop (nth 1 args)))
 				     (list 'math-negp (car args))
-				   (list 'math-lessp
-					 (nth 1 args) (nth 0 args)))))
+				   (cons 'math-lessp args))))
 			  ((eq func '=)
 			   (if (and (math-numberp (nth 1 args))
 				    (math-zerop (nth 1 args)))
