@@ -280,6 +280,13 @@
     (error (math-reject-arg pat (nth 1 err))))
 )
 
+(defun calcFunc-vmatches (expr pat)
+  (condition-case err
+      (or (math-apply-rewrites expr (math-compile-patterns pat))
+	  0)
+    (error (math-reject-arg pat (nth 1 err))))
+)
+
 
 
 ;;; A compiled rule set is an a-list of entries whose cars are functors,
@@ -480,8 +487,7 @@
       (cdr (math-compile-rewrites (cons
 				   'vec
 				   (mapcar (function (lambda (x)
-						       (list 'vec x
-							     '(var XXX XXX))))
+						       (list 'vec x t)))
 					   (if (eq (car-safe pats) 'vec)
 					       (cdr pats)
 					     (list pats))))))))
@@ -625,7 +631,23 @@
 		(setq math-conds (cdr math-conds))
 		(math-rwcomp-cond-instr expr)))
 	    (math-rwcomp-instr 'done
-			       (math-rwcomp-match-vars math-rhs)
+			       (if (eq math-rhs t)
+				   (cons 'vec
+					 (delq
+					  nil
+					  (nreverse
+					   (mapcar
+					    (function
+					     (lambda (v)
+					       (and (car v)
+						    (list
+						     'calcFunc-assign
+						     (math-build-var-name
+						      (car v))
+						     (math-rwcomp-register-expr
+						      (nth 1 v))))))
+					    math-regs))))
+				 (math-rwcomp-match-vars math-rhs))
 			       math-remembering)
 	    (setq math-prog (cdr math-prog))
 	    (let* ((heads (math-rewrite-heads math-pattern))
@@ -1114,6 +1136,7 @@
 		     (math-rwcomp-pattern (car arg1) (cdr arg1))
 		     (math-rwcomp-instr 'try2 try1 (cdr arg2))
 		     (if (and (= part 0) (not def) (not math-rewrite-whole)
+			      (not (eq math-rhs t))
  			      (setq def (get (car expr)
  					     'math-rewrite-default)))
  			 (let ((reg1 (math-rwcomp-reg))
