@@ -67,7 +67,7 @@
 
 (defun calc-select-once-maybe (num)
   (interactive "P")
-  (calc-select-once num t t)
+  (calc-select-here num t t)
 )
 
 (defun calc-select-additional ()
