@@ -1,6 +1,6 @@
 ;; Calculator for GNU Emacs, part II [calc-store.el]
-;; Copyright (C) 1990, 1991 Free Software Foundation, Inc.
-;; Written by Dave Gillespie, daveg@csvax.cs.caltech.edu.
+;; Copyright (C) 1990, 1991, 1992, 1993 Free Software Foundation, Inc.
+;; Written by Dave Gillespie, daveg@synaptics.com.
 
 ;; This file is part of GNU Emacs.
 
@@ -122,7 +122,8 @@
 	(if pop (or calc-store-keep (calc-pop-stack pop)))
 	(calc-record-undo (list 'store (symbol-name var) old))
 	(if tag
-	    (calc-record value (format ">%s%s" tag (calc-var-name var))))
+	    (let ((calc-full-trail-vectors nil))
+	      (calc-record value (format ">%s%s" tag (calc-var-name var)))))
 	(and (memq var '(var-e var-i var-pi var-phi var-gamma))
 	     (eq (car-safe old) 'special-const)
 	     (message "(Note: Built-in definition of %s has been lost)" var))
@@ -363,7 +364,8 @@
 	 (if (eq (car-safe value) 'error)
 	     (error "Bad format in variable contents: %s" (nth 2 value)))
 	 (setq value (calc-normalize value))
-	 (calc-record value (concat "<" (calc-var-name var)))
+	 (let ((calc-full-trail-vectors nil))
+	   (calc-record value (concat "<" (calc-var-name var))))
 	 (calc-push value))))
 )
 
@@ -439,6 +441,16 @@
   (calc-edit-variable 'var-GenCount)
 )
 
+(defun calc-edit-Holidays ()
+  (interactive)
+  (calc-edit-variable 'var-Holidays)
+)
+
+(defun calc-edit-IntegLimit ()
+  (interactive)
+  (calc-edit-variable 'var-IntegLimit)
+)
+
 (defun calc-edit-LineStyles ()
   (interactive)
   (calc-edit-variable 'var-LineStyles)
@@ -464,9 +476,9 @@
   (calc-edit-variable 'var-TimeZone)
 )
 
-(defun calc-edit-UnitSimpRules ()
+(defun calc-edit-Units ()
   (interactive)
-  (calc-edit-variable 'var-UnitSimpRules)
+  (calc-edit-variable 'var-Units)
 )
 
 (defun calc-edit-ExtSimpRules ()
@@ -616,6 +628,9 @@
 		      (not (eq (car-safe (symbol-value x)) 'special-const))
 		      (or (not (eq x 'var-Decls))
 			  (not (equal var-Decls '(vec))))
+		      (or (not (eq x 'var-Holidays))
+			  (not (equal var-Holidays '(vec (var sat var-sat)
+							 (var sun var-sun)))))
 		      (insert "(setq "
 			      (symbol-name x)
 			      " "
