@@ -40,6 +40,27 @@
 							  nil)))))
 )
 
+(defun calc-quick-units ()
+  (interactive)
+  (calc-slow-wrapper
+   (let* ((num (- last-command-char ?0))
+	  (pos (if (= num 0) 10 num))
+	  (units (calc-var-value 'var-Units))
+	  (expr (calc-top-n 1)))
+     (or (and (>= num 0) (<= num 9))
+	 (error "Bad unit number"))
+     (or (math-vectorp units)
+	 (error "No \"quick units\" are defined"))
+     (or (< pos (length units))
+	 (error "Unit number %d not defined" pos))
+     (if (math-units-in-expr-p expr nil)
+	 (calc-enter-result 1 (format "cun%d" num)
+			    (math-convert-units expr (nth pos units)))
+       (calc-enter-result 1 (format "*un%d" num)
+			  (math-simplify-units
+			   (math-mul expr (nth pos units)))))))
+)
+
 (defun calc-convert-units (&optional old-units new-units)
   (interactive)
   (calc-slow-wrapper
@@ -1039,6 +1060,7 @@ Entries are (SYMBOL EXPR DOC-STRING TEMP-TYPE BASE-UNITS).")
 (math-defsimplify /
   (and math-simplifying-units
        (let ((np (cdr expr))
+	     (try-cancel-units 0)
 	     n nn)
 	 (setq n (if (eq (car-safe (nth 2 expr)) '*)
 		     (cdr (nth 2 expr))
@@ -1053,7 +1075,13 @@ Entries are (SYMBOL EXPR DOC-STRING TEMP-TYPE BASE-UNITS).")
 	   (math-simplify-units-divisor (cdr n) (cdr (cdr expr)))
 	   (setq np (cdr (cdr n))))
 	 (math-simplify-units-divisor np (cdr (cdr expr)))
-	 (math-simplify-units-prod)
+	 (if (eq try-cancel-units 0)
+	     (let* ((math-simplifying-units nil)
+		    (base (math-simplify (math-to-standard-units expr nil))))
+	       (if (Math-numberp base)
+		   (setq expr base))))
+	 (if (eq (car-safe expr) '/)
+	     (math-simplify-units-prod))
 	 expr))
 )
 
@@ -1074,11 +1102,34 @@ Entries are (SYMBOL EXPR DOC-STRING TEMP-TYPE BASE-UNITS).")
 
 ;; Simplify, e.g., "in / cm" to "2.54" in a units expression.
 (defun math-simplify-units-quotient (n d)
-  (let ((un (math-check-unit-name n))
-	(ud (math-check-unit-name d)))
-    (and un ud
-	 (equal (nth 4 un) (nth 4 ud))
-	 (math-to-standard-units (list '/ n d) nil)))
+  (let ((pow1 1)
+	(pow2 1))
+    (and (eq (car-safe n) '^)
+	 (integerp (nth 2 n))
+	 (setq pow1 (nth 2 n) n (nth 1 n)))
+    (and (eq (car-safe d) '^)
+	 (integerp (nth 2 d))
+	 (setq pow2 (nth 2 d) d (nth 1 d)))
+    (let ((un (math-check-unit-name n))
+	  (ud (math-check-unit-name d)))
+      (and un ud
+	   (if (and (equal (nth 4 un) (nth 4 ud))
+		    (eq pow1 pow2))
+	       (math-to-standard-units (list '/ n d) nil)
+	     (let (ud1)
+	       (setq un (nth 4 un)
+		     ud (nth 4 ud))
+	       (while un
+		 (setq ud1 ud)
+		 (while ud1
+		   (and (eq (car (car un)) (car (car ud1)))
+			(setq try-cancel-units
+			      (+ try-cancel-units
+				 (- (* (cdr (car un)) pow1)
+				    (* (cdr (car ud)) pow2)))))
+		   (setq ud1 (cdr ud1)))
+		 (setq un (cdr un)))
+	       nil)))))
 )
 
 (math-defsimplify ^
