@@ -1588,11 +1588,36 @@
       (progn
 	(math-read-token)
 	'(vec))
-    (let ((vals (let ((exp-keep-spaces space-sep))
+    (let ((save-exp-pos exp-pos)
+	  (save-exp-old-pos exp-old-pos)
+	  (save-exp-token exp-token)
+	  (save-exp-data exp-data)
+	  (vals (let ((exp-keep-spaces space-sep))
 		  (if (or (equal exp-data "\\dots")
 			  (equal exp-data "\\ldots"))
 		      '(vec (neg (var inf var-inf)))
-		    (math-read-vector)))))
+		    (catch 'syntax (math-read-vector))))))
+      (if (stringp vals)
+	  (if space-sep
+	      (let ((error-exp-pos exp-pos)
+		    (error-exp-old-pos exp-old-pos)
+		    vals2)
+		(setq exp-pos save-exp-pos
+		      exp-old-pos save-exp-old-pos
+		      exp-token save-exp-token
+		      exp-data save-exp-data)
+		(let ((exp-keep-spaces nil))
+		  (setq vals2 (catch 'syntax (math-read-vector))))
+		(if (and (not (stringp vals2))
+			 (or (assoc exp-data '(("\\ldots") ("\\dots") (";")))
+			     (equal exp-data close)
+			     (eq exp-token 'end)))
+		    (setq space-sep nil
+			  vals vals2)
+		  (setq exp-pos error-exp-pos
+			exp-old-pos error-exp-old-pos)
+		  (throw 'syntax vals)))
+	    (throw 'syntax vals)))
       (if (or (equal exp-data "\\dots")
 	      (equal exp-data "\\ldots"))
 	  (progn
