@@ -187,12 +187,12 @@
 	  (goto-char top)
 	  (forward-char (+ (nth 1 vals) (if single 0 1)))
 	  (error (nth 2 vals))))
-    (calc-wrapper
+    (calc-slow-wrapper
      (calc-enter-result 0 "grab" vals)))
 )
 
 
-(defun calc-do-grab-rectangle (top bot arg)
+(defun calc-do-grab-rectangle (top bot arg &optional reduce)
   (and (memq major-mode '(calc-mode calc-trail-mode))
        (error "This command works only in a regular text buffer."))
   (let* ((col1 (save-excursion (goto-char top) (current-column)))
@@ -234,21 +234,24 @@
 		    (setq vals v w 0)
 		  (setq vals (nconc vals (list v))
 			pos j)))))
-	(if (and (null arg)
-		 (string-match "[[{][^][{}]*[]}]" (car data)))
-	    (setq pos (match-beginning 0)
-		  vals (math-read-expr (math-match-substring (car data) 0)))
-	  (let ((s (if (string-match
-			"\\`\\([0-9]+:[ \t]\\)?\\(.*[^, \t]\\)[, \t]*\\'"
-			(car data))
-		       (math-match-substring (car data) 2)
-		     (car data))))
-	    (setq pos -1
-		  vals (math-read-expr (concat "[" s "]")))
-	    (if (eq (car-safe vals) 'error)
-		(let ((v2 (math-read-expr s)))
-		  (or (eq (car-safe v2) 'error)
-		      (setq vals (list 'vec v2))))))))
+	(if (string-match "\\` *-?[0-9][0-9]?[0-9]?[0-9]?[0-9]?[0-9]? *\\'"
+			  (car data))
+	    (setq vals (list 'vec (string-to-int (car data))))
+	  (if (and (null arg)
+		   (string-match "[[{][^][{}]*[]}]" (car data)))
+	      (setq pos (match-beginning 0)
+		    vals (math-read-expr (math-match-substring (car data) 0)))
+	    (let ((s (if (string-match
+			  "\\`\\([0-9]+:[ \t]\\)?\\(.*[^, \t]\\)[, \t]*\\'"
+			  (car data))
+			 (math-match-substring (car data) 2)
+		       (car data))))
+	      (setq pos -1
+		    vals (math-read-expr (concat "[" s "]")))
+	      (if (eq (car-safe vals) 'error)
+		  (let ((v2 (math-read-expr s)))
+		    (or (eq (car-safe v2) 'error)
+			(setq vals (list 'vec v2)))))))))
       (if (eq (car-safe vals) 'error)
 	  (progn
 	    (if calc-was-started
@@ -259,11 +262,15 @@
 	    (forward-line lnum)
 	    (forward-char (+ (nth 1 vals) (min col1 col2) pos))
 	    (error (nth 2 vals))))
-      (setq mat (cons vals mat)
-	    data (cdr data)
+      (or (equal vals '(vec))
+	  (setq mat (cons vals mat)))
+      (setq data (cdr data)
 	    lnum (1+ lnum)))
-    (calc-wrapper
-     (calc-enter-result 0 "grab" (nreverse mat))))
+    (calc-slow-wrapper
+     (if reduce
+	 (calc-enter-result 0 "grb+" (list reduce '(var add var-add)
+					   (nreverse mat)))
+       (calc-enter-result 0 "grab" (nreverse mat)))))
 )
 
 
@@ -279,6 +286,7 @@
        (calc-force-refresh)
        (let ((n (prefix-numeric-value nn))
 	     (eat-lnums calc-line-numbering)
+	     (big-offset (if (eq calc-language 'big) 1 0))
 	     top bot)
 	 (setq oldbuf (current-buffer)
 	       newbuf (or thebuf
@@ -288,7 +296,7 @@
 	 (cond ((and (or (null nn)
 			 (consp nn))
 		     (= (calc-substack-height 0)
-			(1- (calc-substack-height 1))))
+			(- (1- (calc-substack-height 1)) big-offset)))
 		(calc-cursor-stack-index 1)
 		(if (looking-at
 		     (if calc-line-numbering "[0-9]+: *[^ \n]" " *[^ \n]"))
@@ -296,13 +304,12 @@
 		(setq eat-lnums nil
 		      top (point))
 		(calc-cursor-stack-index 0)
-		(setq bot (1- (point))))
+		(setq bot (- (1- (point)) big-offset)))
 	       ((> n 0)
 		(calc-cursor-stack-index n)
 		(setq top (point))
 		(calc-cursor-stack-index 0)
-		(setq bot (- (point)
-			     (if (eq calc-language 'big) 1 0))))
+		(setq bot (- (point) big-offset)))
 	       ((< n 0)
 		(calc-cursor-stack-index (- n))
 		(setq top (point))
@@ -570,9 +577,12 @@ To cancel the edit, simply kill the *Calc Edit* buffer."
 	 (if disp-trail
 	     (calc-trail-display 1 t))
 	 (and vals
-	      (if (>= num 0)
-		  (calc-enter-result num "edit" vals)
-		(calc-enter-result 1 "edit" vals (- num))))))))
+	      (let ((calc-simplify-mode (if (eq last-command-char ?\C-j)
+					    'none
+					  calc-simplify-mode)))
+		(if (>= num 0)
+		    (calc-enter-result num "edit" vals)
+		  (calc-enter-result 1 "edit" vals (- num)))))))))
 )
 
 
